/*
 *  OOo Dictionary Installer (c) Jonathan Hudson 2002.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include <wait.h>
#include <glib.h>

#define ZIPSIG "PK\003\004"
#define ZIPERR (-1)
#define CANERR (-2)

#define AVAIL    "available.lst"
#define HYPHLST  "hyphavail.lst"
#define DICT     "DICT"
#define HYPH     "HYPH"

#define DICTLST  "dictionary.lst"
#define BASEURL  "http://dict.progbits.com"
#define MAX_DICTS 64
//#define UNZIPCMDD "unzip -qq -o %s %s.aff %s.dic  -d %s"
//#define UNZIPCMDH "unzip -qq -o %s %s.dic  -d %s"

#define UNZIPCMDD "unzip -qq -o %s -d %s"
#define UNZIPCMDH "unzip -qq -o %s -d %s"

#define OOTEXT "OpenOffice.org "

#define OOODI_URL "http://ooodi.sourceforge.net"
#define OOODI_HELP "OOodi/doc/OOodi.html"

#define DICT_INSTALLED     (1 << 0)
#define DICT_INSTALLED_S   (1 << 1)
#define DICT_SELECTED      (1 << 2)
#define DICT_DOWNLOADED    (1 << 3)
#define DICT_AVAILABLE     (1 << 4)
#define DICT_REMOVED       (1 << 5)
#define DICT_SPECIAL_LINK  (1 << 6)

#define ZIP_NOCHECK 0
#define ZIP_CHECK   1
#define ZIP_CHECKED 2

#define ADD_NEW 1
#define DEL_DIC 2
#define DEL_ALL 3
#define REPAIR  4

#define IS_DICT 0
#define IS_HYPH 1

#define DICTDATEOFFSET (24*3600*7)

/*
 * The downloader callback
 */

typedef struct
{
    GtkWidget *widget;
    int flag;
    int zflag;
    FILE *fp;
} DLCB_t;

/*
 * The dictionary item type. Holds the data read from available.lst
 * plus flags so we know what to do next
 */

typedef struct
{
    char *lang;
    char *region;
    char *spec;
    char *desc;
    char *file;
    int  flag; 
    int  type;
    int  status;
} DICTITM_t;

/*
 * The dictionary list type. Totals, selected, the OOo path
 * and some useful widgets
 */

typedef struct
{
    int total;
    int iselected;
    int rselected;    
    int s_iselected;
    int s_rselected;
    int location;
    char *path;
    char *path_shr;
    GtkWidget *install;
    GtkWidget *remove;    
    GtkWidget *install_s;
    GtkWidget *remove_s;    
    GtkWidget *clist;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    DICTITM_t item[MAX_DICTS];
} DICTLST_t;


#define USER_LOCATION 0
#define SYSTEM_LOCATION 1

#define USERLOCMASK (1 << USER_LOCATION)
#define SYSLOCMASK (1 << SYSTEM_LOCATION)

typedef struct 
{
    char *name;
    char *region;
    short location;
    short type;
} DICTNAMES;


/*
 * The 'global' data
 */

typedef struct
{
    GtkWidget *window;
    GtkWidget *errwin;
    GtkWidget *popmenu;
    GtkWidget *about;
    GtkWidget *props;
    GtkWidget *warn;
    GtkWidget *copywin;    
    GList *vers;
    char *proxy;
    char *uri;
    char *pxuser;
    char *pxpass;
    char *browser;
    char *help;
    char *basedir;
    char  unlink_on_remove;
    char  keepzips;
    char  uselocal;
    char  usehyph;
    DICTNAMES *dlst;
    int nd, ndmax;
    int flag;
    DICTLST_t avail;
} OOODL_GBL_t;

typedef enum {AVAIL_TYPE, HYPH_TYPE, BASE_TYPE, USER_BASE} FSTYPE_t;
typedef enum {ACT_CONT=-1, ACT_NONE=0, ACT_ED, ACT_DL, ACT_FS, ACT_EX} ACT_t;

typedef struct
{
    FSTYPE_t typ;
    ACT_t act;
    char *fn;
    char *wanted;
} FPDAT_t;

#define OO_FILE_ERR (-2)
#define OO_FILE_UNKNOWN (-1)
#define OO_FILE_CURR (0)
#define OO_FILE_NEW (1)

#define UGLY_HACK 0x80000000

extern int OODownload(char *, char *, int);
extern void  setwinicon (GtkWidget *);
extern  gboolean on_ooo_window_button_press_event (GtkWidget *, GdkEventButton *,
                                           gpointer);
extern void DisplayWarning(GtkWidget *, char *title, char *label);
extern void SaveConfigFile(void);
extern GtkWidget* create_filesel (char *title);
extern int loadhyphdicts(int);
extern int OORemoteStatus (char *, time_t *);
extern int OOFileStatus (char *file, char*, unsigned int);
extern int read_available_file(char *filename, short type, int n);
extern void load_the_lists(void);
extern char * buildpath(char *path,...);

extern OOODL_GBL_t *gblptr;
